"""
CLI
"""
import argparse
from pathlib import Path
from windowsget import download
from windowsget.logger import logger


def main():
    """
    Download a file from a given URL and save it to the specified output file or to the current
    working directory.

    The script takes a URL as input and downloads the file located at that URL to the specified
    output file or to the current working directory if no output file is specified. It uses the
    `windowsget` package to perform the download.

    Example usage:
        python downloader.py https://example.com/file.zip -o /path/to/output.zip
    """
    parser = argparse.ArgumentParser(
        description="Download a file from a given URL and save it to \
        the specified output file or to the current working directory."
    )

    # Add a positional argument for the URL
    parser.add_argument(
        "url", type=str,
        help="The URL of the file to download."
    )

    # Add an optional argument for the output file
    parser.add_argument(
        "--output", "-o",
        help=("The path to save the downloaded file. If not set, the file will be saved to the "
              "current working directory with the same name as in the URL.")
    )

    # Parse the arguments
    args = parser.parse_args()

    # Set the filepath to the output file or to the current working directory
    filepath = args.output
    if filepath is None:
        filepath = Path(args.url).name
    try:
        # Download the file and get the size of the downloaded file
        file_size = download(url=args.url, filepath=Path(filepath))
        # Print the success message with the filepath and size of the downloaded file
        logger.info(f"Download successfully completed at {filepath} of size: {file_size} bytes")
    except Exception as exe:
        logger.exception(exe)
        raise exe


if __name__ == "__main__":
    main()



"""
Downloader module
"""
import os
from pathlib import Path
import requests
from windowsget.logger import logger


def download(url: str, filepath: Path) -> int:
    """
    Download a file from the specified url and save it to the specified filepath.

    Args:
        url (str): The url of the file to download.
        filepath (Path): The file path where to save the downloaded file.

    Returns:
        int: The number of bytes downloaded.

    Raises:
        HTTPError: If the HTTP status code is not 200 (OK).
    """
    response = requests.get(url, timeout=2)

    # Check if the HTTP status code is OK (200)
    if response.status_code == 200:
        # Create the parent directory of the file if it does not exist
        filepath.parent.mkdir(exist_ok=True, parents=True)

        _, extension = os.path.splitext(filepath)
        if extension == "":
            logger.exception("ValuError: The file extension must be provided!")
            raise ValueError("The file extension must be provided!")

        # Write the contents of the response to the file
        with open(filepath, 'wb') as file_handler:
            bytes_downloaded = file_handler.write(response.content)

        # Return the number of bytes downloaded
        return bytes_downloaded
    logger.warning(f"status code: {response.status_code}, raising exception")
    # Raise an HTTPError if the status code is not OK
    response.raise_for_status()
    return 0



"""
Logger module for logging
"""
import os
import logging
import sys

LOGGING_STR = "[%(asctime)s: %(levelname)s: %(module)s]: %(message)s"
LOG_DIR = "./logs"
log_filepath = os.path.join(LOG_DIR, "running_logs.log")
os.makedirs(LOG_DIR, exist_ok=True)

logging.basicConfig(
    level=logging.INFO,
    format=LOGGING_STR,
    handlers=[
        logging.FileHandler(log_filepath),
        logging.StreamHandler(sys.stdout),
    ],
)

logger = logging.getLogger("windowsget")



import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

__version__ = "0.0.5"

REPO_NAME = "windowsget"
AUTHOR_USER_NAME = "c17hawke"
PACKAGE_NAME = "windowsget"
AUTHOR_EMAIL = "sunny.c17hawke@gmail.com"
COMMAND_NAME = "windowsget"

setuptools.setup(
    name=PACKAGE_NAME,
    version=__version__,
    author=AUTHOR_USER_NAME,
    author_email=AUTHOR_EMAIL,
    description="A small python package",
    long_description=long_description,
    long_description_content="text/markdown",
    url=f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}",
    project_urls={
        "Bug Tracker": f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}/issues",
        "Documentation": f"https://{AUTHOR_USER_NAME}.github.io/{REPO_NAME}/",
        "Homepage": f"https://{AUTHOR_USER_NAME}.github.io/{REPO_NAME}/",
        "Source": f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}",
    },
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    entry_points={
        "console_scripts": [
            f"{COMMAND_NAME} = {PACKAGE_NAME}.cli:main",
        ]
    }
)
